#![allow(unused)]

use esp_idf_hal::{
    self,
    i2c::{I2cConfig, I2cDriver},
    prelude::Peripherals,
    units::Hertz,
};
use esp_idf_sys as _; // If using the `binstart` feature of `esp-idf-sys`, always keep this module imported

mod error;
use error::Error;

mod appstate;

mod gps;
use gps::GPS;

mod display;
use display::Display;
use ssd1306::prelude::*;

fn main() -> Result<(), Error> {
    // It is necessary to call this function once. Otherwise some patches to the runtime
    // implemented by esp-idf-sys might not link properly. See https://github.com/esp-rs/esp-idf-template/issues/71
    esp_idf_sys::link_patches();
    // Bind the log crate to the ESP Logging facilities
    //esp_idf_svc::log::EspLogger::initialize_default();

    // Get Peripherals and sysloop ready
    let peripherals = Peripherals::take().unwrap();
    let pins = peripherals.pins;

    // Define program's pins + addresses
    let scl = pins.gpio22;
    let sda = pins.gpio21;
    let gps_rx = pins.gpio16;
    let _lcd_address: u8 = 0x3C;

    // Setup I2C driver
    let config = I2cConfig::new().baudrate(Hertz(921600));
    let i2c =
        I2cDriver::new(peripherals.i2c0, sda, scl, &config).map_err(|err| Error::EspError(err))?;

    // Prepare components for processing 
    let mut gps: GPS = GPS::new(peripherals.uart0, gps_rx)?;
    let mut display: Display<DisplaySize128x64> =
        Display::new(i2c, DisplaySize128x64, DisplayRotation::Rotate0)?;
    let mut _app_state = appstate::AppState::default();

    loop {
        // Get the latest data from GPS module and flush to Display
        match gps.poll() {
            Ok(res) => display.draw(&res.get_speed().as_str())?,
            Err(e) => {
                display.draw("NO\nGPS\nDATA")?;
                eprintln!("{:?}", e);
            }
        }
    }
}
